@@ -155,11 +155,8 @@ theta.des=NULL,inverse=0,step=1)
    return(ud)
 }
 
-prop<-function(x) x
-
-
-"summary.two.stage" <- function (object,digits = 3,...) {
-  if (!(inherits(object, 'two.stage') || inherits(object, 'two.stage.reg'))) stop("Must be a Two-Stage object")
+summary.two.stage<-function (object,digits = 3,...) {
+  if (!(inherits(object, 'two.stage') )) stop("Must be a Two-Stage object")
   
   prop<-TRUE; 
   if (is.null(object$gamma)==TRUE) stop(" No regression terms"); 
@@ -209,7 +206,7 @@ prop<-function(x) x
 }
 
 print.two.stage <- function (x,digits = 3,...) {
-  if (!(inherits(x, 'two.stage') || inherits(x, 'two.stage.reg'))) stop("Must be a Two-Stage object")
+  if (!(inherits(x, 'two.stage') )) stop("Must be a Two-Stage object")
   cat(" Two-stage estimation for Clayton-Oakes-Glidden  model\n"); 
   cat(" Marginals of Cox-Aalen form, dependence by variance of Gamma distribution\n\n");  
   object <- x; rm(x);
@@ -225,15 +222,15 @@ print.two.stage <- function (x,digits = 3,...) {
   print(attr(object,'Call'))
 }
 
-
 coef.two.stage<-function(object,digits=3,d2logl=1,...) {
    coefBase(object,digits=digits,d2logl=d2logl,...)
 }
 
-"plot.two.stage" <- function (x,  pointwise.ci=1, robust=0, specific.comps=FALSE,level=0.05, 
-                              start.time = 0, stop.time = 0, add.to.plot=FALSE,mains=TRUE,
-                              xlab="Time", ylab ="Cumulative regression function",...) {
-  if (!(inherits(x, 'two.stage') || inherits(x, 'two.stage.reg'))) stop("Must be a Two-Stage object")
+plot.two.stage<-function(x,pointwise.ci=1,robust=0,specific.comps=FALSE,
+		level=0.05, 
+		start.time=0,stop.time=0,add.to.plot=FALSE,mains=TRUE,
+                xlab="Time",ylab ="Cumulative regression function",...) {
+  if (!(inherits(x, 'two.stage'))) stop("Must be a Two-Stage object")
   object <- x; rm(x);  
  
   B<-object$cum; V<-object$var.cum; p<-dim(B)[[2]]; 
@@ -264,5 +261,7 @@ coef.two.stage<-function(object,digits=3,d2logl=1,...) {
       lines(B[,1],ul,lty=robust,type="s"); 
       lines(B[,1],nl,lty=robust,type="s"); }
     abline(h=0); 
-  }
 }
+}   
+
+
